#include "activeentry.h"
#include "images.h"
#include "jackctrl.h"
#include "guiconfig.h"

extern global_info gi;
extern GUIconfig *guicfg;
extern Images *appImages;

ActiveEntry::ActiveEntry(ActivePlaylist *p, audio_entry *e, QString n, ChannelPortList cVSp, int v) : QWidget(p),
	layout(new QGridLayout(this, 2, 9, 3)), 
	label_index(new QLabel(this)),
	label_key(new QLabel(this)),
	label_length(new QLabel(this)),
	label_portcols(new QLabel(this)),
	button_stop(new QPushButton(this)),
	dial_vol(new QDial(0, 140, 20, v, this)),
	parent(p),
	vuMeter(new VuMeter(this, e->channels)),
	audioEntry(e), next(0), prev(0), index(parent->itemCount),
	showRemaining(guicfg->showRemaining), hasBeenUpdated(false),
	label_name(new QLabel(n, this)),
	ports(cVSp), totalFrames(0), currentFrame(0) {

	button_stop->setPixmap(*(appImages->pixmap_kill_xpm));
	button_stop->setFlat(true);
	button_stop->setFocusPolicy(QWidget::NoFocus);

	QFont hFont;
	hFont.setWeight(QFont::Bold);
	label_index->setFont(hFont);
	label_name->setFont(hFont);

	dial_vol->setFixedSize(30, 30);
	dial_vol->setNotchesVisible(true);
	dial_vol->setLineStep(20);
	dial_vol->setFocusPolicy(QWidget::NoFocus);

	label_name->setFixedWidth(200);

	label_portcols->setPixmap(getPortCols(this, &ports));

	layout->addMultiCellWidget(label_index, 0, 1, 1, 1, Qt::AlignCenter);
	layout->addMultiCellWidget(label_name, 0, 0, 2, 4);
	layout->addWidget(label_length, 1, 2);
	layout->addWidget(label_key, 1, 3, Qt::AlignHCenter);
	layout->addWidget(label_portcols, 1, 4, Qt::AlignRight);
	layout->addMultiCellWidget(dial_vol, 0, 1, 5, 5);
	layout->addMultiCellWidget(button_stop, 0, 1, 6, 6);
	layout->addMultiCellWidget(vuMeter, 0, 1, 7, 7);

	layout->setColStretch(0, 1);
	layout->setColStretch(1, 0);
	layout->setColStretch(2, 0);
	layout->setColStretch(3, 0);
	layout->setColStretch(4, 0);
	layout->setColStretch(5, 0);
	layout->setColStretch(6, 0);
	layout->setColStretch(7, 0);
	layout->setColStretch(8, 1);

	connect(button_stop, SIGNAL(clicked()), this, SLOT(stop()));
}

ActiveEntry::~ActiveEntry() {
	delete label_index;
	delete label_key;
	delete label_length;
	delete label_portcols;
	delete button_stop;
	delete dial_vol;
	delete vuMeter;
	delete label_name;
	delete layout;
}

void ActiveEntry::updateData() {
	totalFrames = audioEntry->totalFrames;
	currentFrame = audioEntry->currentFrame;
	vuMeter->setData(audioEntry->vu);
	audioEntry->volume = (float) dial_vol->value() / 100.0;
	hasBeenUpdated = true;
}

void ActiveEntry::updateWidgets() {
	label_index->setText("#" + QString::number(index));

	uint minutes, seconds;
	if(showRemaining)
		seconds = (totalFrames - currentFrame) / gi.jack_samplerate;
	else
		seconds = currentFrame / gi.jack_samplerate;

	minutes = seconds / 60;
	seconds %= 60;

	if(showRemaining)
		if(seconds < 10)
			label_length->setText("(-" + QString::number(minutes) + ":0" + QString::number(seconds) + ")");
		else
			label_length->setText("(-" + QString::number(minutes) + ":" + QString::number(seconds) + ")");
	else
		if(seconds < 10)
			label_length->setText("(" + QString::number(minutes) + ":0" + QString::number(seconds) + ")");
		else
			label_length->setText("(" + QString::number(minutes) + ":" + QString::number(seconds) + ")");


	switch(index) {
		case 0:
			label_key->setText(key2string(guicfg->key_kill1));
			break;
		case 1:
			label_key->setText(key2string(guicfg->key_kill2));
			break;
		case 2:
			label_key->setText(key2string(guicfg->key_kill3));
			break;
		case 3:
			label_key->setText(key2string(guicfg->key_kill4));
			break;
		case 4:
			label_key->setText(key2string(guicfg->key_kill5));
			break;
		case 5:
			label_key->setText(key2string(guicfg->key_kill6));
			break;
		case 6:
			label_key->setText(key2string(guicfg->key_kill7));
			break;
		case 7:
			label_key->setText(key2string(guicfg->key_kill8));
			break;
		case 8:
			label_key->setText(key2string(guicfg->key_kill9));
			break;
		case 9:
			label_key->setText(key2string(guicfg->key_kill10));
			break;
		case 10:
			label_key->setText(key2string(guicfg->key_kill11));
			break;
		case 11:
			label_key->setText(key2string(guicfg->key_kill12));
			break;
		default:
			label_key->setText(" - None - ");
	}
}

void ActiveEntry::stop() {
	parent->stop(this);
}

void ActiveEntry::mousePressEvent(QMouseEvent *e) {
	QWidget *clickedOn = childAt(mapFromGlobal(e->globalPos()));

	if(clickedOn == label_length) {
		showRemaining = !showRemaining;
		e->accept();
		return;
	}

	e->ignore();
}

void ActiveEntry::show() {
	QWidget::show();
	resize(sizeHint());
}
