#ifndef COMMON
#define COMMON

#include <sndfile.h>
#include <pthread.h>
#include <jack/jack.h>
#include <jack/ringbuffer.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	int count;
	char **names;
} port_info;

struct _port_connection;

typedef struct _port_connection {
	uint srcChannel;
	char *dstName;
	int last;
} port_connection;


typedef struct {
	const char *filename;
	int error;
	uint totalFrames;
	int samplerate;
	uint channels;
} file_info;

struct _audio_entry;
struct _global_info;

typedef struct _audio_entry {
	uint id;
	struct _global_info *gi;
	char *name;
	int error;
	SF_INFO *sf_info;
	SNDFILE *sf_sndfile;
	jack_ringbuffer_t **ringbuffers;
	port_connection *portConnections;
	int samplerate;
	uint channels;
	uint totalFrames;
	uint currentFrame;
	jack_default_audio_sample_t volume;
	jack_default_audio_sample_t *vu;
	uint count;				// number of repeats left, set by disk_read
	uint eof;				// end-of-file, set by disk_read
	uint eob;				// end-of-buffers, set by jprocess => can be removed from list
	uint fillZero;			// Fill jack buffer with zeros and mark eob next jprocess
	struct _audio_entry *next;
	void *gui_ptr;			// Reserved to GUI. Will point to widget corresponding to the audio entry
} audio_entry;


#define THREAD_STATUS_RWLOCK	1 << 0
#define THREAD_STATUS_SIG_COND	1 << 1
#define THREAD_STATUS_COND_MUT	1 << 2
#define THREAD_STATUS_PTHREAD	1 << 3

typedef struct _global_info {
	// JACKCTRL-related
	jack_client_t *client;
	char *clientName;
	char *internalInterlacedBuffer;
	char *internalBufferArray;
	jack_default_audio_sample_t *internalJackBuffer;
	port_info *jackPortsInfo;
	jack_port_t **jackPorts;
	uint internalBufferSize;
	uint maxAudioChannels;
	uint ringbufferSizeFactor;
	uint jackBufSize;
	uint audioSampleSize;  // = sizeof(jack_default_audio_sample_t)
	uint jack_samplerate;
	audio_entry *audioList;
	uint audioIdCounter;
	
	// MISC
	int debugLevel;

	// Thread and mutexes
	uint thread_status;
	pthread_rwlock_t  rwlock;           // linked audio list; obtain write lock before deleting entry
	pthread_cond_t    signal_condition; // Signal jprocess => disk_read that data is requested
	pthread_mutex_t   condition_mutex;  // Mutex for above  -^-
	pthread_t pthread;
} global_info;



#ifdef __cplusplus
}
#endif

#endif
