#ifndef configwidgetsh
#define configwidgetsh

#include <qgroupbox.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include <qcheckbox.h>

#include "guiconfig.h"

class GrabKey;
class ConfigKeys;
class ConfigPortColors;

enum FuncKeys {ctrl, play, skipnext, alloff, kill1, kill2, kill3, kill4, kill5, kill6, kill7, kill8, kill9, kill10, kill11, kill12};

class ConfigPortColors : public QGroupBox {
	Q_OBJECT
	
	private:
		QGridLayout *layout;
		QLabel *note1, *note2, *labelAvail, *labelMapped;
		QListBox *availPorts, *mappedPorts;
		QPushButton *button;

		GUIconfig *parent;
		bool isHandlingSelChg;
		bool canAdd, canRem;

		QColor getNextColor();
		QColor getColor(int i);
		bool hasColor(QColor col, int index);
		void updateButton();

	public:
		ConfigPortColors(GUIconfig *p);
		~ConfigPortColors();
		void getData();
		void saveData();
		void doAdd();
		void doRem();
		

	public slots:
		void click();
		void sigMappedSelChanged();
		void sigAvailSelChanged();
		void mappedDoubleClicked(QListBoxItem * item);
};

class ConfigKeys : public QGroupBox {
	Q_OBJECT
	
	private:
		QGridLayout *layout;
		QLabel *note1;
		QListBox *listKeys;

		GUIconfig *parent;

		int k_ctrl, k_play, k_skip, k_alloff, k_kill1, k_kill2, k_kill3, k_kill4, k_kill5, k_kill6;
		int k_kill7, k_kill8, k_kill9, k_kill10, k_kill11, k_kill12;

	public:
		ConfigKeys(GUIconfig *p);
		void getData();
		void saveData();
		void updateListbox();
		void setKey(FuncKeys f, int k);

	public slots:
		void clicked(QListBoxItem *i);
		void show();
};

class GrabKey : public QLabel {
	Q_OBJECT

	private:
		ConfigKeys *dataParent;
		FuncKeys key;
		QWidget *parent;

	public:
		GrabKey(ConfigKeys *dp, QWidget *pp);
		void show(FuncKeys k);
		void keyPressEvent(QKeyEvent * e);

		QTimer *timer;

	public slots:
		void cancel();
};

class ListBoxTextNum : public QListBoxText {
	public:
		ListBoxTextNum(const QString &text, FuncKeys f) : QListBoxText(text), fk(f) {  }
		FuncKeys fk;
};

class MiscOpts : public QGroupBox {
	Q_OBJECT
	
	private:
		QListBox *listKeys;
		QCheckBox *checkSaveOnClose, *checkShowRemain;

		GUIconfig *parent;

	public:
		MiscOpts(GUIconfig *p);
		void getData();
		void saveData();
};

#else
	class MiscOpts;
	class GrabKey;
	class ConfigKeys;
	class ConfigPortColors;
#endif
