/****************************************************************
**
** TTtrigger by J�rn Christensen, summer 2005
**
****************************************************************/

#include <qapplication.h>
#include <qfileinfo.h>

#include <signal.h>
#include <getopt.h>
#include <cstdlib>

#include "images.h"
#include "jackctrl.h"
#include "mwindow.h"
#include "guiconfig.h"

void handle(int);

GUIconfig *guicfg;
global_info gi;
Images *appImages;

int main(int argc, char **argv) {
	signal(SIGTERM, handle);
	signal(SIGINT, handle);

	QApplication qa(argc, argv);
	appImages = new Images();
	guicfg = new GUIconfig();

	gi.ringbufferSizeFactor = DEF_JACK_RINGBUF_FACTOR;
	gi.maxAudioChannels = DEF_MAX_AUDIO_CHANNELS;
	gi.internalBufferSize = DEF_INT_BUF_SIZE;
	gi.clientName = DEF_CLIENT_NAME;
	gi.debugLevel = DEF_DEBUG_LEVEL;

	extern int opterr;
	int c, show_usage = 0, longopt_index = 0;
	char *optstring = "hc:f:i:r:vqn:";
	struct option long_options[] = {
		{ "help", 0, 0, 'h' },
		{ "max-channels", 1, 0, 'c' },
		{ "session-file", 1, 0, 'f' },
		{ "int-bufsize", 1, 0, 'i' },
		{ "ring-bufsize", 1, 0, 'r' },
		{ "verbose", 0, 0, 'v' },
		{ "quiet", 0, 0, 'q' },
		{ "jack-name", 1, 0, 'n' },
		{ 0, 0, 0, 0 }
	};

	opterr = 0;

	while ((c = getopt_long (argc, argv, optstring, long_options, &longopt_index)) != -1) {
		switch (c) {
			case 1: //getopt signals end of '-' options
				break;

			case 'h':
				show_usage = 1;
				break;

			case 'c':
				gi.maxAudioChannels = atoi(optarg);
				break;

			case 'f':
				guicfg->fileName = QString(optarg);
				break;

			case 'i':
				gi.internalBufferSize = atoi(optarg);
				break;

			case 'r':
				gi.ringbufferSizeFactor = atoi(optarg);
				break;

			case 'v':
				gi.debugLevel = 2;
				break;

			case 'q':
				gi.debugLevel = 0;
				break;

			case 'n':
				gi.clientName = optarg;
				break;

			default:
				show_usage = 1;
				break;
		}
	}

	if (show_usage) {
		qWarning("TTtrigger v" APPVERSION " by J�rn V. Christensen.\n"
			"Usage: tttrigger [OPTIONS]\n"
			"\n"
			"Options:\n"
			" -h --help                 This text\n"
			" -c --max-channels #       Set max allowed channels in an audio file to #\n"
			"                           It affects how much memory is allocated on start up\n"
			"                           Default: %d\n"
			" -f --session-file <file>  Use <file> as session file\n"
			"                           Default: " DEF_FILE_NAME "\n"
			" -n --jack-name <name>     Set jack client name to <name>\n"
			"                           Default: " DEF_CLIENT_NAME "\n"
			" -i --int-bufsize #        Set internal buffer size to #\n"
			"                           Default: %d\n"
			" -r --ring-bufsize #       Set ringbuffer scale factor\n"
			"                           Ringbuffer size = # * jack internal buffer size\n"
			"                           Default: %d\n"
			" -v --verbose              Be verbose\n"
			" -q --quiet                Be quiet\n",
			DEF_MAX_AUDIO_CHANNELS, DEF_INT_BUF_SIZE, DEF_JACK_RINGBUF_FACTOR);
		exit (1);
	}

	QFileInfo fi(guicfg->fileName);
	if(fi.isDir())
		guicfg->file.setName(fi.absFilePath() + "/" + DEF_FILE_NAME);
	else
		guicfg->file.setName(fi.absFilePath());

	init_jackctrl(&gi);

	MainWindow MW;
	return MW.run();
}

void handle(int) {
	qApp->mainWidget()->close();
}
