#include "misc.h"
#include "guiconfig.h"

extern GUIconfig *guicfg;

bool portVScolor::operator< (const portVScolor & s1) {
	return name < s1.name;
}

bool channelVSport::operator< (const channelVSport & s1) {
	return channel < s1.channel;
}

ListBoxPixmapExt::ListBoxPixmapExt(const QString &text, const QColor &col, bool e) : 
	QListBoxPixmap(QPixmap(10, 10), text),
	active(e), blueText(false), color(col) {
	((QPixmap *) pixmap())->fill(color);
}

void ListBoxPixmapExt::updateColor(QColor c) {
	color = c;
	((QPixmap *) pixmap())->fill(color);
}

void ListBoxPixmapExt::paint(QPainter *p) {
	if(isSelected())
		p->setPen(Qt::white);
	else if(blueText)
		p->setPen(Qt::blue);
	else if(active)
		p->setPen(Qt::black);
	else 
		p->setPen(Qt::darkGray);

	QListBoxPixmap::paint(p);
}

ListBoxTextExt::ListBoxTextExt(const QString &text, bool e) : 
	QListBoxText(text),
	active(e), blueText(false) { }

void ListBoxTextExt::paint(QPainter *p) {
	if(isSelected())
		p->setPen(Qt::white);
	else if(blueText)
		p->setPen(Qt::blue);
	else if(active)
		p->setPen(Qt::black);
	else 
		p->setPen(Qt::darkGray);

	QListBoxText::paint(p);
}

QPixmap getPortCols(QWidget *widget, ChannelPortList *listChnPort) {
	QColor *fillColor;
	QPixmap out;

	int cellWidth = 12;
	int cellHeight = 12;

	if(listChnPort->count()) {
	  	int pixmapWidth = cellWidth * listChnPort->count();
  		out = QPixmap(pixmapWidth + 2, cellHeight + 2);
	  	out.fill(Qt::black);
  
	  	int offset = 1;
  		ChannelPortList::iterator it = listChnPort->begin();
  		PortColorList::iterator colit;
  		QPainter painter(&out);
  		while(it != listChnPort->end()) {
  			colit = guicfg->listPortColor.begin();
  			while(colit != guicfg->listPortColor.end()) {
  				if((*colit).name == (*it).name)
  					break;
  				colit++;
  			}
  
  			if(colit == guicfg->listPortColor.end())
  				fillColor = &(guicfg->unknownPort);
  			else
  				fillColor = &((*colit).color);
  
  			painter.fillRect(offset, 1, cellWidth, cellHeight, QBrush(*fillColor));
  
	  		offset += cellWidth;
  			it++;
  		}
	}
	else {
	  	int pixmapWidth = cellWidth;;
  		out = QPixmap(pixmapWidth + 2, cellHeight + 2);
	  	out.fill(widget, 0, 0);
	}

	return out;
}

QString key2string(int key) {
	switch(key) {
		case 0 :
			return QString(" - None - ");
		case Qt::Key_Escape :
			return QString("Key: Escape");
		case Qt::Key_Tab :
			return QString("Key: Tab");
		case Qt::Key_Backtab :
			return QString("Key: Backtab");
		case Qt::Key_Backspace :
			return QString("Key: Backspace");
		case Qt::Key_Return :
			return QString("Key: Return");
		case Qt::Key_Enter :
			return QString("Key: Enter");
		case Qt::Key_Insert :
			return QString("Key: Insert");
		case Qt::Key_Delete :
			return QString("Key: Delete");
		case Qt::Key_Pause :
			return QString("Key: Pause");
		case Qt::Key_Print :
			return QString("Key: Print");
		case Qt::Key_SysReq :
			return QString("Key: SysReq");
		case Qt::Key_Home :
			return QString("Key: Home");
		case Qt::Key_End :
			return QString("Key: End");
		case Qt::Key_Left :
			return QString("Key: Left");
		case Qt::Key_Up :
			return QString("Key: Up");
		case Qt::Key_Right :
			return QString("Key: Right");
		case Qt::Key_Down :
			return QString("Key: Down");
		case Qt::Key_Prior :
			return QString("Key: Prior");
		case Qt::Key_Next :
			return QString("Key: Next");
		case Qt::Key_Shift :
			return QString("Key: Shift");
		case Qt::Key_Control :
			return QString("Key: Control");
		case Qt::Key_Meta :
			return QString("Key: Meta");
		case Qt::Key_Alt :
			return QString("Key: Alt");
		case Qt::Key_CapsLock :
			return QString("Key: CapsLock");
		case Qt::Key_NumLock :
			return QString("Key: NumLock");
		case Qt::Key_ScrollLock :
			return QString("Key: ScrollLock");
		case Qt::Key_Clear :
			return QString("Key: Clear");
		case Qt::Key_F1 :
			return QString("Key: F1");
		case Qt::Key_F2 :
			return QString("Key: F2");
		case Qt::Key_F3 :
			return QString("Key: F3");
		case Qt::Key_F4 :
			return QString("Key: F4");
		case Qt::Key_F5 :
			return QString("Key: F5");
		case Qt::Key_F6 :
			return QString("Key: F6");
		case Qt::Key_F7 :
			return QString("Key: F7");
		case Qt::Key_F8 :
			return QString("Key: F8");
		case Qt::Key_F9 :
			return QString("Key: F9");
		case Qt::Key_F10 :
			return QString("Key: F10");
		case Qt::Key_F11 :
			return QString("Key: F11");
		case Qt::Key_F12 :
			return QString("Key: F12");
		case Qt::Key_F13 :
			return QString("Key: F13");
		case Qt::Key_F14 :
			return QString("Key: F14");
		case Qt::Key_F15 :
			return QString("Key: F15");
		case Qt::Key_F16 :
			return QString("Key: F16");
		case Qt::Key_F17 :
			return QString("Key: F17");
		case Qt::Key_F18 :
			return QString("Key: F18");
		case Qt::Key_F19 :
			return QString("Key: F19");
		case Qt::Key_F20 :
			return QString("Key: F20");
		case Qt::Key_F21 :
			return QString("Key: F21");
		case Qt::Key_F22 :
			return QString("Key: F22");
		case Qt::Key_F23 :
			return QString("Key: F23");
		case Qt::Key_F24 :
			return QString("Key: F24");
		case Qt::Key_F25 :
			return QString("Key: F25");
		case Qt::Key_F26 :
			return QString("Key: F26");
		case Qt::Key_F27 :
			return QString("Key: F27");
		case Qt::Key_F28 :
			return QString("Key: F28");
		case Qt::Key_F29 :
			return QString("Key: F29");
		case Qt::Key_F30 :
			return QString("Key: F30");
		case Qt::Key_F31 :
			return QString("Key: F31");
		case Qt::Key_F32 :
			return QString("Key: F32");
		case Qt::Key_F33 :
			return QString("Key: F33");
		case Qt::Key_F34 :
			return QString("Key: F34");
		case Qt::Key_F35 :
			return QString("Key: F35");
		case Qt::Key_Super_L :
			return QString("Key: Super_L");
		case Qt::Key_Super_R :
			return QString("Key: Super_R");
		case Qt::Key_Menu :
			return QString("Key: Menu");
		case Qt::Key_Hyper_L :
			return QString("Key: Hyper_L");
		case Qt::Key_Hyper_R :
			return QString("Key: Hyper_R");
		case Qt::Key_Help :
			return QString("Key: Help");
		case Qt::Key_Space :
			return QString("Key: Space");
		case Qt::Key_Exclam :
			return QString("Key: Exclam");
		case Qt::Key_QuoteDbl :
			return QString("Key: QuoteDbl");
		case Qt::Key_NumberSign :
			return QString("Key: NumberSign");
		case Qt::Key_Dollar :
			return QString("Key: Dollar");
		case Qt::Key_Percent :
			return QString("Key: Percent");
		case Qt::Key_Ampersand :
			return QString("Key: Ampersand");
		case Qt::Key_Apostrophe :
			return QString("Key: Apostrophe");
		case Qt::Key_ParenLeft :
			return QString("Key: ParenLeft");
		case Qt::Key_ParenRight :
			return QString("Key: ParenRight");
		case Qt::Key_Asterisk :
			return QString("Key: Asterisk");
		case Qt::Key_Plus :
			return QString("Key: Plus");
		case Qt::Key_Comma :
			return QString("Key: Comma");
		case Qt::Key_Minus :
			return QString("Key: Minus");
		case Qt::Key_Period :
			return QString("Key: Period");
		case Qt::Key_Slash :
			return QString("Key: Slash");
		case Qt::Key_0 :
			return QString("Key: 0");
		case Qt::Key_1 :
			return QString("Key: 1");
		case Qt::Key_2 :
			return QString("Key: 2");
		case Qt::Key_3 :
			return QString("Key: 3");
		case Qt::Key_4 :
			return QString("Key: 4");
		case Qt::Key_5 :
			return QString("Key: 5");
		case Qt::Key_6 :
			return QString("Key: 6");
		case Qt::Key_7 :
			return QString("Key: 7");
		case Qt::Key_8 :
			return QString("Key: 8");
		case Qt::Key_9 :
			return QString("Key: 9");
		case Qt::Key_Colon :
			return QString("Key: Colon");
		case Qt::Key_Semicolon :
			return QString("Key: Semicolon");
		case Qt::Key_Less :
			return QString("Key: Less");
		case Qt::Key_Equal :
			return QString("Key: Equal");
		case Qt::Key_Greater :
			return QString("Key: Greater");
		case Qt::Key_Question :
			return QString("Key: Question");
		case Qt::Key_At :
			return QString("Key: At");
		case Qt::Key_A :
			return QString("Key: A");
		case Qt::Key_B :
			return QString("Key: B");
		case Qt::Key_C :
			return QString("Key: C");
		case Qt::Key_D :
			return QString("Key: D");
		case Qt::Key_E :
			return QString("Key: E");
		case Qt::Key_F :
			return QString("Key: F");
		case Qt::Key_G :
			return QString("Key: G");
		case Qt::Key_H :
			return QString("Key: H");
		case Qt::Key_I :
			return QString("Key: I");
		case Qt::Key_J :
			return QString("Key: J");
		case Qt::Key_K :
			return QString("Key: K");
		case Qt::Key_L :
			return QString("Key: L");
		case Qt::Key_M :
			return QString("Key: M");
		case Qt::Key_N :
			return QString("Key: N");
		case Qt::Key_O :
			return QString("Key: O");
		case Qt::Key_P :
			return QString("Key: P");
		case Qt::Key_Q :
			return QString("Key: Q");
		case Qt::Key_R :
			return QString("Key: R");
		case Qt::Key_S :
			return QString("Key: S");
		case Qt::Key_T :
			return QString("Key: T");
		case Qt::Key_U :
			return QString("Key: U");
		case Qt::Key_V :
			return QString("Key: V");
		case Qt::Key_W :
			return QString("Key: W");
		case Qt::Key_X :
			return QString("Key: X");
		case Qt::Key_Y :
			return QString("Key: Y");
		case Qt::Key_Z :
			return QString("Key: Z");
		case Qt::Key_BracketLeft :
			return QString("Key: BracketLeft");
		case Qt::Key_Backslash :
			return QString("Key: Backslash");
		case Qt::Key_BracketRight :
			return QString("Key: BracketRight");
		case Qt::Key_AsciiCircum :
			return QString("Key: AsciiCircum");
		case Qt::Key_Underscore :
			return QString("Key: Underscore");
		case Qt::Key_QuoteLeft :
			return QString("Key: QuoteLeft");
		case Qt::Key_BraceLeft :
			return QString("Key: BraceLeft");
		case Qt::Key_Bar :
			return QString("Key: Bar");
		case Qt::Key_BraceRight :
			return QString("Key: BraceRight");
		case Qt::Key_AsciiTilde :
			return QString("Key: AsciiTilde");
		case Qt::Key_nobreakspace :
			return QString("Key: nobreakspace");
		case Qt::Key_exclamdown :
			return QString("Key: exclamdown");
		case Qt::Key_cent :
			return QString("Key: cent");
		case Qt::Key_sterling :
			return QString("Key: sterling");
		case Qt::Key_currency :
			return QString("Key: currency");
		case Qt::Key_yen :
			return QString("Key: yen");
		case Qt::Key_brokenbar :
			return QString("Key: brokenbar");
		case Qt::Key_section :
			return QString("Key: section");
		case Qt::Key_diaeresis :
			return QString("Key: diaeresis");
		case Qt::Key_copyright :
			return QString("Key: copyright");
		case Qt::Key_ordfeminine :
			return QString("Key: ordfeminine");
		case Qt::Key_guillemotleft :
			return QString("Key: guillemotleft");
		case Qt::Key_notsign :
			return QString("Key: notsign");
		case Qt::Key_hyphen :
			return QString("Key: hyphen");
		case Qt::Key_registered :
			return QString("Key: registered");
		case Qt::Key_macron :
			return QString("Key: macron");
		case Qt::Key_degree :
			return QString("Key: degree");
		case Qt::Key_plusminus :
			return QString("Key: plusminus");
		case Qt::Key_twosuperior :
			return QString("Key: twosuperior");
		case Qt::Key_threesuperior :
			return QString("Key: threesuperior");
		case Qt::Key_acute :
			return QString("Key: acute");
		case Qt::Key_mu :
			return QString("Key: mu");
		case Qt::Key_paragraph :
			return QString("Key: paragraph");
		case Qt::Key_periodcentered :
			return QString("Key: periodcentered");
		case Qt::Key_cedilla :
			return QString("Key: cedilla");
		case Qt::Key_onesuperior :
			return QString("Key: onesuperior");
		case Qt::Key_masculine :
			return QString("Key: masculine");
		case Qt::Key_guillemotright :
			return QString("Key: guillemotright");
		case Qt::Key_onequarter :
			return QString("Key: onequarter");
		case Qt::Key_onehalf :
			return QString("Key: onehalf");
		case Qt::Key_threequarters :
			return QString("Key: threequarters");
		case Qt::Key_questiondown :
			return QString("Key: questiondown");
		case Qt::Key_Agrave :
			return QString("Key: Agrave");
		case Qt::Key_Aacute :
			return QString("Key: Aacute");
		case Qt::Key_Acircumflex :
			return QString("Key: Acircumflex");
		case Qt::Key_Atilde :
			return QString("Key: Atilde");
		case Qt::Key_Adiaeresis :
			return QString("Key: Adiaeresis");
		case Qt::Key_Aring :
			return QString("Key: Aring");
		case Qt::Key_AE :
			return QString("Key: AE");
		case Qt::Key_Ccedilla :
			return QString("Key: Ccedilla");
		case Qt::Key_Egrave :
			return QString("Key: Egrave");
		case Qt::Key_Eacute :
			return QString("Key: Eacute");
		case Qt::Key_Ecircumflex :
			return QString("Key: Ecircumflex");
		case Qt::Key_Ediaeresis :
			return QString("Key: Ediaeresis");
		case Qt::Key_Igrave :
			return QString("Key: Igrave");
		case Qt::Key_Iacute :
			return QString("Key: Iacute");
		case Qt::Key_Icircumflex :
			return QString("Key: Icircumflex");
		case Qt::Key_Idiaeresis :
			return QString("Key: Idiaeresis");
		case Qt::Key_ETH :
			return QString("Key: ETH");
		case Qt::Key_Ntilde :
			return QString("Key: Ntilde");
		case Qt::Key_Ograve :
			return QString("Key: Ograve");
		case Qt::Key_Oacute :
			return QString("Key: Oacute");
		case Qt::Key_Ocircumflex :
			return QString("Key: Ocircumflex");
		case Qt::Key_Otilde :
			return QString("Key: Otilde");
		case Qt::Key_Odiaeresis :
			return QString("Key: Odiaeresis");
		case Qt::Key_multiply :
			return QString("Key: multiply");
		case Qt::Key_Ooblique :
			return QString("Key: Ooblique");
		case Qt::Key_Ugrave :
			return QString("Key: Ugrave");
		case Qt::Key_Uacute :
			return QString("Key: Uacute");
		case Qt::Key_Ucircumflex :
			return QString("Key: Ucircumflex");
		case Qt::Key_Udiaeresis :
			return QString("Key: Udiaeresis");
		case Qt::Key_Yacute :
			return QString("Key: Yacute");
		case Qt::Key_THORN :
			return QString("Key: THORN");
		case Qt::Key_ssharp :
			return QString("Key: ssharp");
		case Qt::Key_agrave :
			return QString("Key: agrave");
		case Qt::Key_aacute :
			return QString("Key: aacute");
		case Qt::Key_acircumflex :
			return QString("Key: acircumflex");
		case Qt::Key_atilde :
			return QString("Key: atilde");
		case Qt::Key_adiaeresis :
			return QString("Key: adiaeresis");
		case Qt::Key_aring :
			return QString("Key: aring");
		case Qt::Key_ae :
			return QString("Key: ae");
		case Qt::Key_ccedilla :
			return QString("Key: ccedilla");
		case Qt::Key_egrave :
			return QString("Key: egrave");
		case Qt::Key_eacute :
			return QString("Key: eacute");
		case Qt::Key_ecircumflex :
			return QString("Key: ecircumflex");
		case Qt::Key_ediaeresis :
			return QString("Key: ediaeresis");
		case Qt::Key_igrave :
			return QString("Key: igrave");
		case Qt::Key_iacute :
			return QString("Key: iacute");
		case Qt::Key_icircumflex :
			return QString("Key: icircumflex");
		case Qt::Key_idiaeresis :
			return QString("Key: idiaeresis");
		case Qt::Key_eth :
			return QString("Key: eth");
		case Qt::Key_ntilde :
			return QString("Key: ntilde");
		case Qt::Key_ograve :
			return QString("Key: ograve");
		case Qt::Key_oacute :
			return QString("Key: oacute");
		case Qt::Key_ocircumflex :
			return QString("Key: ocircumflex");
		case Qt::Key_otilde :
			return QString("Key: otilde");
		case Qt::Key_odiaeresis :
			return QString("Key: odiaeresis");
		case Qt::Key_division :
			return QString("Key: division");
		case Qt::Key_oslash :
			return QString("Key: oslash");
		case Qt::Key_ugrave :
			return QString("Key: ugrave");
		case Qt::Key_uacute :
			return QString("Key: uacute");
		case Qt::Key_ucircumflex :
			return QString("Key: ucircumflex");
		case Qt::Key_udiaeresis :
			return QString("Key: udiaeresis");
		case Qt::Key_yacute :
			return QString("Key: yacute");
		case Qt::Key_thorn :
			return QString("Key: thorn");
		case Qt::Key_ydiaeresis :
			return QString("Key: ydiaeresis");
		default:
			return QString("Key: -unknown-");
	}
}
