#ifndef playlisth
#define playlisth

#include <qwidget.h>
#include <qlayout.h>
#include <qscrollbar.h>
#include <qstring.h>
#include <qevent.h>
#include <qptrlist.h>
#include <qtimer.h>

#include "misc.h"
#include "playlistbase.h"
#include "playlistentry.h"
#include "portselector.h"
#include "jumpto.h"
#include "popupname.h"
#include "popupmenu.h"

class Playlist : public QWidget {
	Q_OBJECT

	private:
		QScrollBar *scroll;

		QWidget *parent;

		bool isUpdatingGraphics;

	public:
		Playlist(QWidget *p, QScrollBar *scb);
		~Playlist();

		PortQuickSelect *portSelectMenu;
		AdvPortSelect *portAdvanced;
		WidgetJumpTo *widgetJumpTo;
		PopupName *popupName;
		PopupMenu *popupMenu;

		PlaylistEntry *ptrCurrent, *ptrPrev, *ptrNext;
		PlaylistBase *itemFirst, *itemLast;
		uint itemCount;
		bool keyModPressed;

		void insertSep(QString name, PlaylistBase *entry = 0);
		bool openEntry(PlaylistEntry *entry);
		void openAddEntry(QString filename, QString name, ChannelPortList chnVSport, int h, int v, int r);
		void updateButtonGraphics();
		void moveItem(PlaylistBase *item);
		void popupNameEdit(PlaylistBase *current, bool next);
		void addEntry(PlaylistBase *entry);
		void remEntry(PlaylistBase *entry);
		void remFromList(PlaylistBase *entry);

		void setCurrent(PlaylistBase *entry);
		void currentAdvance();
		void currentBack();
		PlaylistEntry * currentGetNext(PlaylistEntry * offset);
		PlaylistEntry * currentGetPrev(PlaylistEntry * offset);		
		void fixPrevNext();
		void setCurrentFirstEnabled();

		void resizeEvent (QResizeEvent *);
		int getMinWidth();

	public slots:
		void updateGraphics();
		void addSep(PlaylistBase *entry = 0);
		void removeAll(bool confirm = true);
		void updateScroll();
		void updateLayout();
		void getFilename_s(PlaylistBase *n = 0, bool updateItem = true);
};
#else
	class Playlist;
#endif
