#include <qapplication.h>

#include "images.h"
#include "guiconfig.h"

#include "playlistbase.h"

extern GUIconfig *guicfg;
extern Images *appImages;

PlaylistBase::PlaylistBase(Playlist *p) : QWidget(p),
	parent(p), next(0), prev(0), errorFlag(0), isEntry(false) {
	layout = new QHBoxLayout(this, 3);

	index = parent->itemCount;

	label_current = new QLabel(this);
	label_index = new QLabel(this);
	label_name = new QLabel(this);
	button_moveDown = new QPushButton(this);
	button_moveUp = new QPushButton(this);;

	label_current->setPixmap(*(appImages->pixmap_position_blank_xpm));
	button_moveDown->setPixmap(*(appImages->pixmap_down_xpm));
	button_moveUp->setPixmap(*(appImages->pixmap_up_xpm));

	connect(button_moveDown, SIGNAL(clicked()), this, SLOT(moveDown()));
	connect(button_moveUp, SIGNAL(clicked()), this, SLOT(moveUp()));
}


PlaylistBase::~PlaylistBase() {
	delete label_current;
	delete label_index;
	delete label_name;
	delete button_moveDown;
	delete button_moveUp;
	delete layout;
}

bool PlaylistBase::isEnabled() {
	return false;
}


QString PlaylistBase::saveText() {
	QString out("[-Separator-]\n");

	if(!label_name->text().isEmpty())
		out += "Name = " + label_name->text() + "\n";

	return out;
}

void PlaylistBase::updateButtonGraphics() {
	if(parent->keyModPressed) {
		button_moveDown->setPixmap(*(appImages->pixmap_down5_xpm));
		button_moveUp->setPixmap(*(appImages->pixmap_up5_xpm));
	}
	else {
		button_moveDown->setPixmap(*(appImages->pixmap_down_xpm));
		button_moveUp->setPixmap(*(appImages->pixmap_up_xpm));
	}
}

void PlaylistBase::compareSize(PBaseWidth *width) {
	if(width->width_index < label_index->sizeHint().width())
		width->width_index = label_index->sizeHint().width();

	//ignore width_name, width_center
}

void PlaylistBase::adjustSize(PBaseWidth *width) {
	label_index->setFixedWidth(width->width_index);

	//ignore width_name, width_center
}

void PlaylistBase::updateWidgets() {
	label_index->setText("#" + QString::number(index));
} 

void PlaylistBase::moveUp() {
	if(index == 0)
		return;

	if(parent->keyModPressed) {
		if(index > 5)
			index -= 5;
		else
			index = 0;
	}
	else{
		if(index > 1)
			index--;
		else
			index = 0;
	}

	parent->moveItem(this);
}

void PlaylistBase::moveDown() {
	if(index == parent->itemCount - 1)
		return;

	if(parent->keyModPressed) {
		index += 5;
	}
	else{
		index++;
	}

	parent->moveItem(this);
}

void PlaylistBase::mousePressEvent(QMouseEvent *e) {
	QWidget *clickedOn = childAt(mapFromGlobal(e->globalPos()));

	if(clickedOn == label_name) {
		if(parent->keyModPressed) {
			e->accept();
			parent->popupName->popup(this);
			return;
		}
	}
	else if(clickedOn == label_index) {
		if(parent->keyModPressed)
			parent->widgetJumpTo->popup(e->globalPos(), this);
		else if(e->button() & Qt::RightButton)
			parent->popupMenu->popup(e->globalPos(), this);

		e->accept();
		return;
	}
	else if(clickedOn == label_current) {
		if(parent->keyModPressed) {
			parent->setCurrent(this);
			e->accept();
			return;
		}
		else if(e->button() & Qt::RightButton) {
			parent->popupMenu->popup(e->globalPos(), this);
			e->accept();
			return;
		}
	}

	e->ignore();
}

void PlaylistBase::mouseDoubleClickEvent(QMouseEvent *e) {
	QWidget *clickedOn = childAt(mapFromGlobal(e->globalPos()));

	if(clickedOn == label_name) {
		if(e->button() & Qt::LeftButton) {
			e->accept();
			parent->popupName->popup(this);
			return;
		}
	}
	else if(clickedOn == label_index) {
		if(e->button() & Qt::LeftButton) {
			parent->widgetJumpTo->popup(e->globalPos(), this);
			e->accept();
			return;
		}
	}
	else if(clickedOn == label_current) {
		if(e->button() & Qt::LeftButton) {
			parent->setCurrent(this);
			e->accept();
			return;
		}
	}

	e->ignore();
}
