#include <qapplication.h>
#include <qfileinfo.h>

#include "popupname.h"
#include "playlistentry.h"
#include "config.h"
#include "guiconfig.h"

extern GUIconfig *guicfg;

PopupName::PopupName(Playlist *p) : QPopupMenu(p), parent(p) {
	data = new QLineEdit(this);
	connect(data, SIGNAL(returnPressed()), this, SLOT(hide()));
	connect(data, SIGNAL(lostFocus()), this, SLOT(hide()));
	insertItem(data);
	minWidth = sizeHint().width();
}

void PopupName::popup(PlaylistBase *dtP) {
	dataParent = dtP;
	dataValid = true;

	int w = dtP->label_name->width() < minWidth ? minWidth : dtP->label_name->width();
	data->setFixedWidth(w);
	setFixedWidth(w);
	data->setText(dtP->label_name->text());

	QPopupMenu::popup(dtP->mapToGlobal(dtP->label_name->pos()));

	data->setFocus();
	data->selectAll();
}

void PopupName::hide() {
	QPopupMenu::hide();
	if(dataValid) {
		QString newName = data->text().stripWhiteSpace();
		if(newName.isEmpty()) {
			if(dataParent->isEntry) {
				PlaylistEntry *e = ((PlaylistEntry *) dataParent);
				QFileInfo qfileinfo(e->filename);
				e->label_name->setText(qfileinfo.baseName());
			}
			else
				dataParent->label_name->setText("- Unnamed -");
		}
		else
			dataParent->label_name->setText(newName);

		parent->updateGraphics();
	}
}

void PopupName::keyPressEvent(QKeyEvent *e) {
	if(e->ascii() == 27) {
		dataValid = false;
		hide();
	}

	if(e->key() == Qt::Key_Up) {
		hide();
		parent->popupNameEdit(dataParent, false);
	}
	else if(e->key() == Qt::Key_Down) {
		hide();
		parent->popupNameEdit(dataParent, true);
	}

	if(e->key() == guicfg->key_ctrl) {
		parent->keyModPressed = true;
		parent->updateButtonGraphics();
	}

	QPopupMenu::keyPressEvent(e);
}

void PopupName::keyReleaseEvent(QKeyEvent * e){
	if(e->key() == guicfg->key_ctrl) {
		parent->keyModPressed = false;
		parent->updateButtonGraphics();
	}

	QPopupMenu::keyReleaseEvent(e);
}

bool PopupName::focusNextPrevChild(bool next) {
	hide();
	parent->popupNameEdit(dataParent, next);
	return true;
}

