#include <qapplication.h>
#include <qsize.h>
#include <qrect.h>
#include <qmessagebox.h>
#include <qpointarray.h>

#include "portselector.h"
#include "guiconfig.h"
#include "config.h"
#include "jackctrl.h"

extern GUIconfig *guicfg;
extern global_info gi;

PortQuickSelect::PortQuickSelect(Playlist *p) : QPopupMenu(p), parent(p) { }


void PortQuickSelect::popup(QPoint point, PlaylistEntry *dtP) {
	dataParent = dtP;

	QuickSelectItem *none = new QuickSelectItem(this, QColor(), " - None - ", lNone); 
	insertItem(none);
	insertSeparator();

	PortColorList::iterator colit = guicfg->listPortColor.begin();
	QSIList::iterator childsit;
	while(colit != guicfg->listPortColor.end()) {
		childsit = childs.begin();
		while(childsit != childs.end()) {
			if((*colit).color == ((QuickSelectItem *) (*childsit))->color)
				break;

			childsit++;
		}

		if(childsit == childs.end()) {           // New color
			QuickSelectItem *t = new QuickSelectItem(this, (*colit).color, (*colit).name);
			childs.push_back(t);
		}
		else                                     // Known color
			((QuickSelectItem *) (*childsit))->addPort((*colit).name);

		colit++;
	}


	childsit = childs.begin();
	while(childsit != childs.end()) {
		insertItem(((QuickSelectItem *) (*childsit)));
		childsit++;
	}

	insertSeparator();

	QuickSelectItem *adv = new QuickSelectItem(this, QColor(), "Advanced...", lAdvanced);
	insertItem(adv);
	
	QPopupMenu::popup(point);
}

void PortQuickSelect::selected(QStringList data) {
	dataParent->listChnPort.clear();
	channelVSport entry;

	if(data.count()) {
		QStringList::iterator it = data.begin();
		for(entry.channel = 0; entry.channel < dataParent->channels; entry.channel++) {
			entry.name = (*it);
			dataParent->listChnPort.push_back(entry);

			it++;
			if(it == data.end())
				it = data.begin();
		}
	}

	hide();

	parent->updateGraphics();
}

void PortQuickSelect::selectNone() {
	dataParent->listChnPort.clear();
	hide();
	parent->updateGraphics();
}

void PortQuickSelect::showAdvanced() {
	if(parent->keyModPressed) {
		parent->keyModPressed = false;
		parent->updateButtonGraphics();
	}

	parent->portAdvanced->show(dataParent);
}

void PortQuickSelect::hide() {
	QPopupMenu::hide();
	clear();
	childs.clear();
}

void PortQuickSelect::keyPressEvent(QKeyEvent * e) {
	if(e->key() == guicfg->key_ctrl) {
		parent->keyModPressed = true;
		parent->updateButtonGraphics();
	}

	QPopupMenu::keyPressEvent(e);
}

void PortQuickSelect::keyReleaseEvent(QKeyEvent * e){
	if(e->key() == guicfg->key_ctrl) {
		parent->keyModPressed = false;
		parent->updateButtonGraphics();
	}

	QPopupMenu::keyReleaseEvent(e);
}


// ******************************************************************************************


QuickSelectItem::QuickSelectItem(PortQuickSelect *p, QColor col, QString portname, LabelType t) : 
	QWidget(0), parent(p), color(col), labelType(t) {
	textData.push_back(portname);
	pixmap = new QPixmap(1, 1);
	pixmap_label = new QLabel(this);
	layout = new QGridLayout(this, 1, 2, 1, 0);
	layout->setColStretch(0, 0);
	layout->setColStretch(1, 1);

	setMouseTracking(true);
	setFocusPolicy(QWidget::StrongFocus);
}

QuickSelectItem::~QuickSelectItem() {
	delete pixmap_label;
	delete pixmap;
	delete layout;
}

void QuickSelectItem::mousePressEvent(QMouseEvent *) {
	switch(labelType) {
		case lNone:
			parent->selectNone();
			return;
		case lAdvanced:
			parent->showAdvanced();
			return;
		default:
			parent->selected(textData);
	}
}

void QuickSelectItem::mouseReleaseEvent(QMouseEvent *) {
	switch(labelType) {
		case lNone:
			parent->selectNone();
			return;
		case lAdvanced:
			parent->showAdvanced();
			return;
		default:
			parent->selected(textData);
	}
}

void QuickSelectItem::enterEvent(QEvent *) {
	setFocus();
}

void QuickSelectItem::leaveEvent(QEvent *) {
	clearFocus();
}

void QuickSelectItem::show() {
	QLabel *l;
	uint i, pixmapHeight = 0;


	if(color.isValid()) {
		qHeapSort(textData);

    	QStringList::iterator it = textData.begin();
    	i = 0;
    	while(it != textData.end()) {
    		l = new QLabel((*it), this);
    		layout->addWidget(l, i++, 1, Qt::AlignLeft);
    		pixmapHeight += l->sizeHint().height();
    		labels.push_back(l);
    		it++;
    	}
    
    	pixmap->resize(pixmapWidth, pixmapHeight);
    	pixmap->fill(color);
    	pixmap_label->setPixmap(*pixmap);
    	layout->addMultiCellWidget(pixmap_label, 0, textData.count() - 1, 0, 0);
	}
	else {
   		l = new QLabel(textData.first(), this);
		labels.push_back(l);
   		layout->addMultiCellWidget(l, 0, 0, 0, 1, Qt::AlignCenter);

		delete pixmap;
		pixmap = 0;
		delete pixmap_label;
		pixmap_label = 0;

	}

	QWidget::show();
}

void QuickSelectItem::addPort(QString portname) {
	textData.push_back(portname);
}

void QuickSelectItem::focusInEvent(QFocusEvent *) {
	QPalette pal;
	setBackgroundMode(Qt::PaletteHighlight);

	QLabelList::iterator it = labels.begin();
	while(it != labels.end()) {
		((QLabel *) (*it))->setBackgroundMode(Qt::PaletteHighlight);
		pal = ((QLabel *) (*it))->palette();
		pal.setColor(QColorGroup::Foreground, Qt::white);
		((QLabel *) (*it))->setPalette(pal);
		it++;
	}

	update();
}

void QuickSelectItem::focusOutEvent(QFocusEvent *) {
	QPalette pal;
	setBackgroundMode(Qt::PaletteBackground);


	QLabelList::iterator it = labels.begin();
	while(it != labels.end()) {
		((QLabel *) (*it))->setBackgroundMode(Qt::PaletteBackground);
		pal = ((QLabel *) (*it))->palette();
		pal.setColor(QColorGroup::Foreground, Qt::black);
		((QLabel *) (*it))->setPalette(pal);
		it++;
	}

	update();
}

// ******************************************************************************************


ConnectionLines::ConnectionLines(QWidget *p, QListBox *ch, QListBox *po, ChannelPortList *d) : QWidget(p),
	channels(ch), ports(po), parent(p), data(d) { }

void ConnectionLines::paintEvent(QPaintEvent *) {
	QListBoxItem *from, *to;
	QPainter painter(this);
	painter.setPen(Qt::black);

	ChannelPortList::iterator it = data->begin();
	while(it != data->end()) {
		from = channels->findItem("Channel " + QString::number((*it).channel), Qt::CaseSensitive | Qt::ExactMatch);
		if(from) {
			to = ports->findItem((*it).name, Qt::CaseSensitive | Qt::ExactMatch);
			if(to) {
				drawLine(&painter, from, to);
			}
		}

		it++;
	}
}

void ConnectionLines::drawLine(QPainter *painter, QListBoxItem *from, QListBoxItem *to) {
	QPoint offset = mapToGlobal(QPoint(0, 0));
	int y, topIndex, index, midPoint = width() / 2;
	QRect topRect;
	QPointArray points(4);

	// I assume that all items has same height (same font anyway)
	topIndex = channels->topItem();
	topRect = channels->itemRect(channels->item(topIndex));
	index = channels->index(from);
	y = channels->mapToGlobal(topRect.topLeft()).y() + topRect.height() / 2 - offset.y() + 
							(index - topIndex) * topRect.height();


	points.setPoint(0, 0, y);
	points.setPoint(1, midPoint, y);

	// I assume that all items has same height (same font anyway)
	topIndex = ports->topItem();
	topRect = ports->itemRect(ports->item(topIndex));
	index = ports->index(to);
	y = ports->mapToGlobal(topRect.topLeft()).y() + topRect.height() / 2 - offset.y() + 
							(index - topIndex) * topRect.height();

	points.setPoint(2, midPoint, y);
	points.setPoint(3, width(), y);

	painter->drawCubicBezier(points);
}

// ******************************************************************************************

AdvPortSelect::AdvPortSelect(Playlist *p) : QWidget(p, 0, WType_Dialog | WStyle_DialogBorder | WShowModal), parent(p) {
	layout = new QGridLayout(this, 3, 3,3);

	channels = new QListBox(this);
	ports = new QListBox(this);	
	cnLines = new ConnectionLines(this, channels, ports, &data);

	channels->setFixedWidth(85);
	channels->setFixedHeight(132);
	ports->setFixedWidth(160);
	ports->setFixedHeight(132);
	cnLines->setFixedWidth(140);
	cnLines->setFixedHeight(132);

	btOk = new QPushButton("OK", this);
	btCancel = new QPushButton("Cancel", this);
	doAddRem = new QPushButton(this);

	layout->addWidget(channels, 0, 0);
	layout->addWidget(cnLines, 0, 1);
	layout->addWidget(ports, 0, 2);
	layout->addWidget(doAddRem, 1, 1, Qt::AlignCenter);
	layout->addWidget(btOk, 2, 0, Qt::AlignLeft);
	layout->addWidget(btCancel, 2, 2, Qt::AlignRight);

	connect(btOk, SIGNAL(clicked()), this, SLOT(ok()));
	connect(btCancel, SIGNAL(clicked()), this, SLOT(hide()));
	connect(doAddRem, SIGNAL(clicked()), this, SLOT(connDisc()));
	connect(channels, SIGNAL(selectionChanged(QListBoxItem *)), this, SLOT(selectChannels(QListBoxItem *)));
	connect(ports, SIGNAL(selectionChanged(QListBoxItem *)), this, SLOT(selectPorts(QListBoxItem *)));
	connect(channels, SIGNAL(clicked(QListBoxItem *)), this, SLOT(selectChannels(QListBoxItem *)));
	connect(ports, SIGNAL(clicked(QListBoxItem *)), this, SLOT(selectPorts(QListBoxItem *)));
	connect(channels, SIGNAL(contentsMoving(int, int)), cnLines, SLOT(update()));
	connect(ports, SIGNAL(contentsMoving(int, int)), cnLines, SLOT(update()));

	layout->setResizeMode(QLayout::Fixed);
}

AdvPortSelect::~AdvPortSelect() {
	delete cnLines;
	delete channels;
	delete ports;
	delete btOk;
	delete btCancel;
	delete doAddRem;
	delete layout;
}

void AdvPortSelect::show(PlaylistEntry *e) {
	char **itPorts;
	PortColorList::iterator itList;
	ChannelPortList::iterator itChn = data.begin();
	QListBoxItem *item;

	entry = e;
	setCaption("TTtrigger - Port connection for " + entry->label_name->text());
	data = entry->listChnPort;

	channels->clear();
	for(uint i = 0; i < entry->channels; i++)
		channels->insertItem(new ListBoxTextExt("Channel " + QString::number(i)));

	// Update ports - get jack ports
	ports->clear();
	port_info *jackports = gi.jackPortsInfo;
	if(jackports) {
		itPorts = (char **) jackports->names;
		while(*itPorts) {
			itList = guicfg->listPortColor.begin();
			while(itList != guicfg->listPortColor.end()) {
				if((*itList).name == *itPorts)
					break;
				itList++;
			}

			if(itList == guicfg->listPortColor.end())
				ports->insertItem(new ListBoxPixmapExt(QString(*itPorts), guicfg->unknownPort));
			else
				ports->insertItem(new ListBoxPixmapExt(QString(*itPorts), (*itList).color));

			itPorts++;
		}
	}
	else
		QMessageBox::warning(this, "Jack communication", "An error happened while requesting information about jack's ports" , QMessageBox::Ok, QMessageBox::NoButton);
	

	// Update ports - add ports from guiconfig
	itList = guicfg->listPortColor.begin();
	while(itList != guicfg->listPortColor.end()) {
		item = ports->findItem((*itList).name, Qt::CaseSensitive | Qt::ExactMatch);
		if(item == 0)
			ports->insertItem(new ListBoxPixmapExt((*itList).name, (*itList).color, false));

		itList++;
	}


	// Update ports - add ports from audio entry
	itChn = data.begin();
	while(itChn != data.end()) {
		item = ports->findItem((*itChn).name, Qt::CaseSensitive | Qt::ExactMatch);
		if(item == 0)
			ports->insertItem(new ListBoxPixmapExt((*itChn).name, guicfg->unknownPort, false));

		item = channels->findItem("Channel " + QString::number((*itChn).channel), Qt::CaseSensitive | Qt::ExactMatch);
		if(item == 0)
			channels->insertItem(new ListBoxTextExt("Channel " + QString::number((*itChn).channel), false));

		itChn++;
	}


	doAddRem->setText(" =!= ");
	doAddRem->setEnabled(false);


	QWidget::show();
}

void AdvPortSelect::keyPressEvent (QKeyEvent *e) {
	if (e->ascii() == 27)
		btCancel->animateClick();
	else if (e->ascii() == 13)
		btOk->animateClick();
	else
		e->ignore();
}

void AdvPortSelect::ok() {
	entry->listChnPort = data;
	parent->updateGraphics();
	hide();
}

void AdvPortSelect::updateButton() {
	if(channels->selectedItem() == 0 || ports->selectedItem() == 0) {
		doAddRem->setText(" =!= ");
		doAddRem->setEnabled(false);
		return;
	}

	doAddRem->setEnabled(true);

	uint intFrom = channels->selectedItem()->text().mid(8).toUInt();
	QString strTo = ports->selectedItem()->text();

	for(ChannelPortList::iterator it =  data.begin(); it != data.end(); it++)
		if((*it).channel == intFrom)
			if((*it).name == strTo) {
				doAddRem->setText("&Disconnect");
				return;
			}

	doAddRem->setText("&Connect");
}

void AdvPortSelect::selectChannels(QListBoxItem *i) {
	uint portNum = ((ListBoxTextExt *) i)->text().mid(8).toUInt();
	QListBoxItem *item;
	uint j;

	for(j = 0; j < ports->count(); j++)
		((ListBoxPixmapExt *) ports->item(j))->blueText = false;

	ChannelPortList::iterator it = data.begin();
	while(it != data.end()) {
		if((*it).channel == portNum) {
			item = ports->findItem((*it).name, Qt::CaseSensitive | Qt::ExactMatch);
			if(item)
				((ListBoxPixmapExt *) item)->blueText = true;
		}

		it++;
	}

	ports->triggerUpdate(false);

	updateButton();
}

void AdvPortSelect::selectPorts(QListBoxItem *i) {
	QString text = ((ListBoxPixmapExt *) i)->text();
	QListBoxItem *item;

	for(uint j = 0; j < channels->count(); j++)
		((ListBoxTextExt *) channels->item(j))->blueText = false;

	ChannelPortList::iterator it = data.begin();
	while(it != data.end()) {
		if((*it).name == text) {
			item = channels->findItem("Channel " + QString::number((*it).channel), Qt::CaseSensitive | Qt::ExactMatch);
			if(item)
				((ListBoxTextExt *) item)->blueText = true;
		}

		it++;
	}
	
	channels->triggerUpdate(false);

	updateButton();
}

void AdvPortSelect::connDisc() {
	if(channels->selectedItem() == 0 || ports->selectedItem() == 0)
		return;

	uint intFrom = channels->selectedItem()->text().mid(8).toUInt();
	QString strTo = ports->selectedItem()->text();

	for(ChannelPortList::iterator it =  data.begin(); it != data.end(); it++)
		if((*it).channel == intFrom)
			if((*it).name == strTo) {
				data.remove(it);
				cnLines->update();
				doAddRem->setText("&Connect");
				return;
			}

	channelVSport entry;
	entry.channel = intFrom;
	entry.name = strTo;
	data.push_back(entry);
	cnLines->update();
	doAddRem->setText("&Disconnect");
}
