#ifndef portselectorh
#define portselectorh

#include <qwidget.h>
#include <qpopupmenu.h>
#include <qvaluelist.h>
#include <qcolor.h>
#include <qevent.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpoint.h>
#include <qpainter.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qsize.h>

#include "playlist.h"
#include "playlistentry.h"
#include "misc.h"

class PortQuickSelect;
class QuickSelectItem;

enum LabelType {lDefault, lNone, lAdvanced};

typedef QValueList<QuickSelectItem *> QSIList;
typedef QValueList<QLabel *> QLabelList;

class PortQuickSelect : public QPopupMenu {
	Q_OBJECT

	private:
		QSIList childs;
		PlaylistEntry *dataParent;
		Playlist *parent;

	public:
		PortQuickSelect(Playlist *p);
		void selected(QStringList data);

		void popup(QPoint point, PlaylistEntry *dtP);

		void keyPressEvent(QKeyEvent * e);
		void keyReleaseEvent (QKeyEvent * e);

	public slots:
		void showAdvanced();
		void selectNone();
		void hide();
};

class QuickSelectItem : public QWidget {
	Q_OBJECT
	
	private:
		static const int pixmapWidth = 10;

		QGridLayout *layout;
		QLabel *pixmap_label;
		QPixmap *pixmap;
		QLabelList labels;

		QStringList textData;
		PortQuickSelect *parent;

	protected:
		void mousePressEvent(QMouseEvent *);
		void mouseReleaseEvent(QMouseEvent *);
		void enterEvent(QEvent *);
		void leaveEvent(QEvent *);
		void focusInEvent(QFocusEvent *);
		void focusOutEvent(QFocusEvent *);

	public:
		QuickSelectItem(PortQuickSelect *p, QColor col, QString portname, LabelType t = lDefault);
		~QuickSelectItem();

		void addPort(QString portname);

		QColor color;
		LabelType labelType;

	public slots:
		void show();
};

class ConnectionLines : public QWidget {
	Q_OBJECT

	private:
		QListBox *channels, *ports;
		QWidget *parent;
		ChannelPortList *data;

	public:
		ConnectionLines(QWidget *p, QListBox *ch, QListBox *po, ChannelPortList *d);
		void paintEvent(QPaintEvent *);
		void drawLine(QPainter *painter, QListBoxItem *from, QListBoxItem *to);
};

class AdvPortSelect : public QWidget {
	Q_OBJECT

	private:
		ConnectionLines *cnLines;
		QListBox *channels, *ports;
		PlaylistEntry *entry;
		Playlist *parent;
		QGridLayout *layout;

		QPushButton *btOk, *btCancel, *doAddRem;

		ChannelPortList data;

	public:
		AdvPortSelect(Playlist *p);
		~AdvPortSelect();
		void show(PlaylistEntry *e);
		void keyPressEvent (QKeyEvent *e);
		void updateButton();

	public slots:
		void ok();
		void connDisc();
		void selectChannels(QListBoxItem *i);
		void selectPorts(QListBoxItem *i);
};

#else
	class ConnectionLines;
	class AdvPortSelect;
	class PortQuickSelect;
	class QuickSelectItem;
#endif
