#include "vu.h"
#include "images.h"
#include "config.h"
#include <cstdlib>

extern Images *appImages;

VuMeter::VuMeter(ActiveEntry *p, int num) : QWidget(p),
	numOfData(num),
	buffer(new QPixmap(sizeHint())),
	parent(p), 
	data((float *) malloc(sizeof(float) * num)), 
	peak(false), timer(new QTimer(this)) {
	connect(timer, SIGNAL(timeout()), SLOT(updateGraphic()));
	timer->start(TIMER_VU_DELAY);
	setFixedSize(sizeHint());

	setBackgroundMode(Qt::NoBackground);

	for(int i = 0; i < num; i++)
		*(data + i) = 0.0;
}

VuMeter::~VuMeter() {
	free(data);
	delete buffer;
	delete timer;
}

void VuMeter::setData(float *d) {
	if(isVisible()) {
		for(uint i = 0; i < numOfData; i++) {
			if(*(d + i) > *(data + i))
				*(data + i) = *(d + i);

			*(d + i) = 0.0;
		}
	}
	else
		for(uint i = 0; i < numOfData; i++)
			*(d + i) = 0.0;
}

void VuMeter::paintEvent(QPaintEvent *) {
	int drawWidth, offset;
	QPainter p;
	p.begin(buffer);

	QBrush b(Qt::black);

	p.fillRect(rect(), Qt::black);

	for(uint i = 0; i < numOfData; i++) {
		offset = 1 + 8 * i;

		if(*(data + i) >= 1.0) {
			peak = true;
			drawWidth = 160;
		}
		else {
			drawWidth = (int) (150.0 * *(data + i));
			if(peak)
				p.drawPixmap (151, offset, *(appImages->pixmap_vu_xpm), 150, 0, 10);
		}

		p.drawPixmap (1, offset, *(appImages->pixmap_vu_xpm), 0, 0, drawWidth);
	}

	p.end();

	bitBlt(this, 0, 0, buffer);
}

void VuMeter::updateGraphic() {
	if(!isVisible())
		return;

	for(uint i = 0; i < numOfData; i++)
		*(data + i) *= 0.9;

	update();
}

void VuMeter::mousePressEvent(QMouseEvent *) {
	peak = false;
}

QSize VuMeter::sizeHint() {
		return QSize(162, 1 + 8 * numOfData);
}
